from setuptools import setup, find_packages

setup(
    name='psrx',
    version='1.3',
    packages=find_packages(),
     entry_points={
        'console_scripts': [
            'psrx = srx_classes.main_script:main',
        ],
    },
    install_requires=[
        '2to3==1.0',
        'altgraph==0.17.4',
        'argcomplete==3.1.2',
        'bcrypt==4.0.1',
        'cffi==1.16.0',
        'click==8.1.7',
        'clipboard==0.0.4',
        'colorama==0.4.6',
        'cryptography==41.0.4',
        'ecdsa==0.18.0',
        'et-xmlfile==1.1.0',
        'gmpy2==2.1.5',
        'Jinja2==3.1.2',
        'junos-eznc==2.6.8',
        'lxml==4.9.3',
        'markdown-it-py==3.0.0',
        'MarkupSafe==2.1.3',
        'mdurl==0.1.2',
        'ncclient==0.6.13',
        'netaddr==0.9.0',
        'numpy==1.26.1',
        'openpyxl==3.1.2',
        'packaging==23.2',
        'pandas==2.1.1',
        'paramiko==3.3.1',
        'pefile==2023.2.7',
        'progressbar==2.5',
        'prompt-toolkit==3.0.21',
        'pycparser==2.21',
        'Pygments==2.16.1',
        'pyinstaller==6.0.0',
        'pyinstaller-hooks-contrib==2023.9',
        'PyNaCl==1.5.0',
        'pyparsing==3.1.1',
        'pyperclip==1.8.2',
        'pyreadline==2.1',
        'pyserial==3.5',
        'python-dateutil==2.8.2',
        'pytz==2023.3.post1',
        'pywin32-ctypes==0.2.2',
        'PyYAML==6.0.1',
        'regex==2023.10.3',
        'rich==13.6.0',
        'scp==0.14.5',
        'shellingham==1.5.3',
        'six==1.16.0',
        'termcolor==2.3.0',
        'tqdm==4.66.1',
        'transitions==0.9.0',
        'typer==0.9.0',
        'typing_extensions==4.8.0',
        'tzdata==2023.3',
        'wcwidth==0.2.8',
        'XlsxWriter==3.1.9',
        'yamlordereddictloader==0.4.2',
    ],
    author='Eslam',
    description='doing all configuration by one click',
    long_description=open('README.md').read(),
    license='MIT',
)
